% A code to perform a simple data reduction on data collected from a CCD.
% James Bebbington

% INPUT =   -   a FITS file name (eg. IMG476.FIT)
%           -   date images were taken 
% OUTPUT =  -   a master bias and master flatfield 
%           -   a reduced FITS file (eg. from above, named: reduced_IMG476.FIT)
%           -   4 seperate FITS files representing filtered red, green and
%               blue data. (eg. from above, the blue will be named:
%               Breduced_IMG476.FIT)

clear all;
clc;

%% Data acquisition.

chosen_date = input('What date were your images taken on, in format eg. 13-Oct-2012. ','s');
% The user is asked for the date they took their images on, to be used to 
% identify the flat fields and bias frames they took. 

filename = input('Enter the full name of the file you wish to reduce. ','s');
% The user will be asked to input their filename in order for the script
% to perform the image reduction process.

raw_data = fitsread(filename); 
% Reads the FITS data from given file name. The raw data will contain many
% imperfections such as dust on the scope and rogue pixels.

decision = input('Now enter (1) if you have bias and flatfield frames from the same day as your science image, (2) if you have master bias and flatfield frames from another date, (3) if you wish to use the default master frames. ','s');

%%

if decision == '1'
% The following lines of code calculate specifics of the data; the standard 
% deviation of pixel value and the maximum pixel value. The program then 
% cycles through the images corresponding to the date specified by the user
% to check for their existence and then sets criteria to distinguish 
% between bias and flat-field images. The output is a master bias and 
% flatfield which is the median of the combination of bias and flatfields 
% taken on the date specified. 


F = [];
B = [];

disp('Now locating and combining bias frames for use in a master bias frame.')

for q = 1:2000
    Q = num2str(q);
    file = strcat('IMG',Q,'.FIT');
    A = exist(file,'file');
    if A == 2;
        info = fitsinfo(file);
        date = info.FileModDate(1:11);
        if date == chosen_date_b
            pixels = fitsread(file);
            pix_analysis = pixels(pixels~=0);
            maxpix = max(pix_analysis(:));
            minpix = min(pix_analysis(:));
            range = maxpix - minpix;
            if range < 1000  && size(B,3) < 3
                B = cat(3,B,pixels);
                disp(file); disp(' used in master bias.');   
            end
        end
    end
end

master_bias = (median(B,3));
biasfile = strcat(chosen_date,'_masterbias');
fitswrite2(transpose(master_bias),biasfile);
% Bias frames are exposures whilst the telescope shutter is closed with a
% duration of 0 seconds. They correct for imperfections such as dust.

disp('Now locating and combining flat-field frames for a master flat.');
    
for q = 1:2000
    Q = num2str(q);
    file = strcat('IMG',Q,'.FIT');
    A = exist(file,'file');
    if A == 2;
        info = fitsinfo(file);
        date = info.FileModDate(1:11);
        if date == chosen_date
            pixels = fitsread(file);
            devpix = std(pixels(:));
            maxpix = max(pixels(:));
            medpix = median(pixels(:));
            if devpix > 1000 && medpix > 5000 && medpix < 40000 && size(F,3) < 3
                pixels = pixels - master_bias;
                pixels = pixels/medpix;    
                F = cat(3,F,pixels);
                disp(file); disp(' used in master flat.');
            end
        end
    end
end

master_flatfield = (median(F,3));
flatfile = strcat(chosen_date,'_masterflat');
fitswrite2(transpose(master_flatfield),flatfile);
% The flatfield is used to correct for pixel-to-pixel variations within the 
% CCD response. It is a uniformly illuminated image, an example of this is 
% the twilight sky. 

%% Data reduction.

elseif decision == '2'
     
     biasfile = input('Enter the name of the master bias you wish to use. ','s');
     flatfile = input('Enter the name of the master flat you wish to use. ','s');
     master_bias = fitsread(biasfile);
     master_flatfield = fitsread(flatfile);

elseif decision == '3'
    
    master_bias = fitsread('default_bias.FIT');
    master_flatfield = fitsread('default_flat.FIT');
    
end

disp('Now commencing data reduction.');
 
reduced_image = (raw_data - master_bias)./(master_flatfield);
% The bias frame data is subtracted from the raw data and then this is
% divided by a flatfield frame.

filename2 = strcat('reduced_',filename); 
fitswrite2(transpose(reduced_image),filename2);
% These final two lines will write the modified, reduced data to another
% fits file using the function fitswrite2, Author: R. G. Abraham, Institute
% of Astronomy, Cambridge University, abraham@ast.cam.ac.uk

%% Seperation of data into R,G and B. 

disp('Separating image into its R,G and B constituents.');


    B = reduced_image(1:2:end,1:2:end);
    G1 = reduced_image(2:2:end,1:2:end);
    G2 = reduced_image(1:2:end,2:2:end);
    R = reduced_image(2:2:end,2:2:end);

% If the user decides they would like to seperate the image out, the
% program will pick out all the different colours from the Bayer matrix
% mosaic and store them into 4 seperate arrays.  

disp('4 matrices returned: B = Blue, R = Red, G1 & G2 = Green.');

Bfile = strcat('B',filename2);
Rfile = strcat('R',filename2);
G1file = strcat('G_one',filename2);
G2file = strcat('G_two',filename2);
% Defines new file names for each different colour. 

fitswrite2(transpose(B),Bfile);
fitswrite2(transpose(R),Rfile);
fitswrite2(transpose(G1),G1file);
fitswrite2(transpose(G2),G2file);
% Writes the different colour images as FITS files.  

disp('4 FITS files returned.');

choice = input('Do you have more images to reduce via this method? Y/N ','s');

while choice == 'Y'
    
filename = input('Enter the full name of the file you wish to reduce. ','s');
% The user will be asked to input their filename in order for the script
% to perform the image reduction process.

raw_data = fitsread(filename); 
% Reads the FITS data from given file name. The raw data will contain many
% imperfections such as dust on the scope and rogue pixels.  
    
    disp('Now commencing data reduction.');
 
reduced_image = (raw_data - master_bias)./(master_flatfield);
% The bias frame data is subtracted from the raw data and then this is
% divided by a flatfield frame.

filename2 = strcat('reduced_',filename); 
fitswrite2(transpose(reduced_image),filename2);
% These final two lines will write the modified, reduced data to another
% fits file using the function fitswrite2, Author: R. G. Abraham, Institute
% of Astronomy, Cambridge University, abraham@ast.cam.ac.uk 

disp('Separating image into its R,G and B constituents.');

    B = reduced_image(1:2:end,1:2:end);
    G1 = reduced_image(2:2:end,1:2:end);
    G2 = reduced_image(1:2:end,2:2:end);
    R = reduced_image(2:2:end,2:2:end);

% If the user decides they would like to seperate the image out, the
% program will pick out all the different colours from the Bayer matrix
% mosaic and store them into 4 seperate arrays.  

disp('4 matrices returned: B = Blue, R = Red, G1 & G2 = Green.');

Bfile = strcat('B',filename2);
Rfile = strcat('R',filename2);
G1file = strcat('G_one',filename2);
G2file = strcat('G_two',filename2);
% Defines new file names for each different colour. 

fitswrite2(transpose(B),Bfile);
fitswrite2(transpose(R),Rfile);
fitswrite2(transpose(G1),G1file);
fitswrite2(transpose(G2),G2file);
% Writes the different colour images as FITS files.  

disp('4 FITS files returned.');

choice = input('Do you have more images to reduce via this method? Y/N ','s');

end

